@@ -150,6 +150,8 @@ static std::vector<SP<CSessionDevice>> scanGPUs(SP<CBackend> backend) {
             continue;
         }
 
+        sessionDevice->resolveMatchingRenderNode(device);
+
         udev_device_unref(device);
 
         if (isBootVGA)
@@ -588,7 +590,7 @@ bool Aquamarine::CDRMBackend::shouldBlit() {
 bool Aquamarine::CDRMBackend::initMgpu() {
     SP<CGBMAllocator> newAllocator;
     if (primary || backend->primaryAllocator->type() != AQ_ALLOCATOR_TYPE_GBM) {
-        newAllocator            = CGBMAllocator::create(backend->reopenDRMNode(gpu->fd), backend);
+        newAllocator            = CGBMAllocator::create(backend->reopenDRMNode(gpu->renderNodeFd >= 0 ? gpu->renderNodeFd : gpu->fd), backend);
         rendererState.allocator = newAllocator;
     } else {
         newAllocator            = ((CGBMAllocator*)backend->primaryAllocator.get())->self.lock();
@@ -906,6 +908,10 @@ int Aquamarine::CDRMBackend::drmFD() {
     return gpu->fd;
 }
 
+int Aquamarine::CDRMBackend::drmRenderNodeFD() {
+    return gpu->renderNodeFd;
+}
+
 static void handlePF(int fd, unsigned seq, unsigned tv_sec, unsigned tv_usec, unsigned crtc_id, void* data) {
     auto pageFlip = (SDRMPageFlip*)data;
 
@@ -969,7 +975,7 @@ void Aquamarine::CDRMBackend::onReady() {
     // init a drm renderer to gather gl formats.
     // if we are secondary, initMgpu will have done that
     if (!primary) {
-        auto a = CGBMAllocator::create(backend->reopenDRMNode(gpu->fd), backend);
+        auto a = CGBMAllocator::create(backend->reopenDRMNode(gpu->renderNodeFd >= 0 ? gpu->renderNodeFd : gpu->fd), backend);
         if (!a)
             backend->log(AQ_LOG_ERROR, "drm: onReady: no renderer for gl formats");
         else {
